(*************************************************************************

     $RCSfile: OLPrefsGUI.mod $
  Description: Implements a GUI for OLPrefs using EAGUI.library.

   Created by: fjc (Frank Copeland)
    $Revision: 1.2 $
      $Author: fjc $
        $Date: 1995/06/29 19:27:47 $

  Copyright  1995, Frank Copeland
  This module forms part of the OL program
  See OLPrefs.doc for conditions of use and distribution

  Log entries are at the end of the file.

*************************************************************************)

<* STANDARD- *> <* MAIN- *>

MODULE OLPrefsGUI;

IMPORT
  SYS := SYSTEM, Kernel, Errors, str := Strings, s := Sets, e := Exec,
  u := Utility, d := Dos, gfx := Graphics, i := Intuition, gt := GadTools,
  ea := EAGUI, eam := EAGUI_Macros, spt := EASupport, lbl := EALabels,
  sep := EASeperators, frm := EAFrames, ead := EADialogs, OLPrefsRev,
  OLSettings, cat := OLPrefsStrings, Events, ASL, ip := IntuiPointer,
  wb := Workbench, du := DosUtil, Icon;

(* ---------------------------------- *)

CONST

  (* Gadget IDs *)

  PrefsFileID    =   0;
  SymSearchID    =   1;
  ObjSearchID    =   2;
  WithPathID     =   3;
  ProgPathID     =   4;
  SymExtID       =   5;
  ObjExtID       =   6;
  WithExtID      =   7;
  LinkCmdID      =   8;
  LinkArgsID     =   9;
  VerboseID      =  10;
  MakeIconsID    =  11;
  WithFmtID      =  12;

  BrowsePrefsID  = 101;
  BrowseLinkerID = 102;
  AboutID        = 103;
  SaveID         = 104;
  SaveAsID       = 105;
  QuitID         = 106;

(* ---------------------------------- *)

TYPE

  SettingsDialog = POINTER TO SettingsDialogRec;
  SettingsDialogRec = RECORD (ead.EADialogRec)
    label  : ARRAY 6 OF lbl.Label;
    frame  : ARRAY 7 OF frm.Frame;
    gadget : ARRAY 13 OF i.GadgetPtr;
    withFmt : LONGINT;
  END; (* SettingsDialogRec *)


(* ---------------------------------- *)

VAR

  scr : i.ScreenPtr;
  visualInfo : gt.VisualInfo;
  drawInfo : i.DrawInfoPtr;
  textAttr : gfx.TextAttr;
  settingsDlg : SettingsDialog;
  hSpace, vSpace : LONGINT;
  tags1, tags2 : ARRAY 2 OF u.TagItem;
  txt : ARRAY 4 OF e.LSTRPTR;


(* ---------------------------------- *)
PROCEDURE Notice* ( msg : LONGINT );

  VAR es : i.EasyStruct; text : e.LSTRPTR;

  BEGIN (* Notice *)
    es.structSize := SIZE (i.EasyStruct);
    es.flags := {};
    es.title := cat.GetString (cat.gui2);
    es.gadgetFormat := cat.GetString (cat.gui4);
    es.textFormat := SYS.ADR ("%s");
    text := cat.GetString (msg);
    IF i.EasyRequest ( NIL, SYS.ADR (es), NIL, text ) = 0
    THEN
    END
  END Notice;

(* ---------------------------------- *)
PROCEDURE Notice1* ( fmt : LONGINT; msg : ARRAY OF CHAR );

  VAR es : i.EasyStruct; text : e.LSTRPTR;

  <*$CopyArrays-*>
  BEGIN (* Notice1 *)
    es.structSize := SIZE (i.EasyStruct);
    es.flags := {};
    es.title := cat.GetString (cat.gui2);
    es.gadgetFormat := cat.GetString (cat.gui4);
    es.textFormat := cat.GetString (fmt);
    IF i.EasyRequest ( NIL, SYS.ADR (es), NIL, SYS.ADR (msg) ) = 0
    THEN
    END
  END Notice1;

(* ---------------------------------- *)
PROCEDURE RequestFile
  ( window     : i.WindowPtr;
    title      : ARRAY OF CHAR;
    pattern    : ARRAY OF CHAR;
    VAR drawer : ARRAY OF CHAR;
    VAR file   : ARRAY OF CHAR;
    save       : BOOLEAN )
  : BOOLEAN;

  VAR
    fr : ASL.FileRequesterPtr;
    flags : s.SET32;
    result : BOOLEAN;

  <*$CopyArrays-*>
  BEGIN (* RequestFile *)
    IF save THEN flags := {ASL.frDoPatterns, ASL.frDoSaveMode}
    ELSE flags := {ASL.frDoPatterns}
    END;
    fr := SYS.VAL (ASL.FileRequesterPtr,
      ASL.AllocAslRequestTags ( ASL.fileRequest,
          ASL.window,         window,
          ASL.titleText,      SYS.ADR (title),
          ASL.initialDrawer,  SYS.ADR (drawer),
          ASL.initialFile,    SYS.ADR (file),
          ASL.initialPattern, SYS.ADR (pattern),
          ASL.flags1,         flags,
          ASL.flags2,         {ASL.frRejectIcons},
          u.end ));
    IF fr # NIL THEN
      result := ASL.AslRequestTags (fr, u.end);
      IF result THEN
        COPY (fr.drawer^, drawer);
        COPY (fr.file^, file)
      END;
      ASL.FreeAslRequest (fr)
    ELSE
      result := FALSE
    END;

    RETURN result
  END RequestFile;

(*------------------------------------*)
PROCEDURE MakeIcon ( file : ARRAY OF CHAR );

  CONST defPrefsIcon = "def_prefs";

  VAR
    icon    : ARRAY 256 OF CHAR;
    diskObj : wb.DiskObjectPtr;
    oldTool : e.LSTRPTR;

  <*$CopyArrays-*>
  BEGIN (* MakeIcon *)
    ASSERT (Icon.base # NIL, 100);
    COPY (file, icon); str.Append (".info", icon);
    IF ~du.FileExists (icon) THEN
      diskObj := Icon.GetDiskObject ("ENV:OLPrefs/def_prefs");
      IF diskObj = NIL THEN
        diskObj := Icon.GetDefDiskObject (wb.project)
      END;
      IF diskObj # NIL THEN
        oldTool := diskObj.defaultTool;
        diskObj.defaultTool := SYS.ADR ("OLPrefs");
        diskObj.currentX := wb.noIconPosition;
        diskObj.currentY := wb.noIconPosition;
        IF ~Icon.PutDiskObject (file, diskObj) THEN
          Notice1 (cat.gui9, icon)
        END;
        diskObj.defaultTool := oldTool;
        Icon.FreeDiskObject (diskObj)
      ELSE
        Notice (cat.gui10)
      END
    END
  END MakeIcon;

(* ---------------------------------- *)
PROCEDURE (dlg : SettingsDialog) SetAttrs;

  BEGIN (* SetAttrs *)
    gt.SetGadgetAttrs ( dlg.gadget[PrefsFileID]^, dlg.win, NIL,
        gt.txText, SYS.ADR (OLSettings.PrefsFile),
        u.done );
    gt.SetGadgetAttrs ( dlg.gadget[SymSearchID]^, dlg.win, NIL,
        gt.stString, SYS.ADR (OLSettings.SymSearch),
        u.done );
    gt.SetGadgetAttrs ( dlg.gadget[ObjSearchID]^, dlg.win, NIL,
        gt.stString, SYS.ADR (OLSettings.ObjSearch),
        u.done );
    gt.SetGadgetAttrs ( dlg.gadget[WithPathID]^, dlg.win, NIL,
        gt.stString, SYS.ADR (OLSettings.WithPath),
        u.done );
    gt.SetGadgetAttrs ( dlg.gadget[ProgPathID]^, dlg.win, NIL,
        gt.stString, SYS.ADR (OLSettings.ProgPath),
        u.done );
    gt.SetGadgetAttrs ( dlg.gadget[SymExtID]^, dlg.win, NIL,
        gt.stString, SYS.ADR (OLSettings.SymExt),
        u.done );
    gt.SetGadgetAttrs ( dlg.gadget[ObjExtID]^, dlg.win, NIL,
        gt.stString, SYS.ADR (OLSettings.ObjExt),
        u.done );
    gt.SetGadgetAttrs ( dlg.gadget[WithExtID]^, dlg.win, NIL,
        gt.stString, SYS.ADR (OLSettings.WithExt),
        u.done );
    gt.SetGadgetAttrs ( dlg.gadget[LinkCmdID]^, dlg.win, NIL,
        gt.stString, SYS.ADR (OLSettings.LinkCmd),
        u.done );
    gt.SetGadgetAttrs ( dlg.gadget[LinkArgsID]^, dlg.win, NIL,
        gt.stString, SYS.ADR (OLSettings.LinkArgs),
        u.done );
    gt.SetGadgetAttrs ( dlg.gadget[VerboseID]^, dlg.win, NIL,
        gt.cbChecked, OLSettings.Verbose,
        u.done );
    gt.SetGadgetAttrs ( dlg.gadget[MakeIconsID]^, dlg.win, NIL,
        gt.cbChecked, OLSettings.MakeIcons,
        u.done );
    gt.SetGadgetAttrs ( dlg.gadget[WithFmtID]^, dlg.win, NIL,
        gt.mxActive,  OLSettings.WithFmt,
        u.done );
    dlg.withFmt := OLSettings.WithFmt;
  END SetAttrs;

(* ---------------------------------- *)
PROCEDURE (dlg : SettingsDialog) GetAttrs;

  PROCEDURE GetString ( gad : i.GadgetPtr; VAR str : ARRAY OF CHAR );

    VAR stringInfo : i.StringInfoPtr;

    BEGIN (* GetString *)
      stringInfo := SYS.VAL (i.StringInfoPtr, gad.specialInfo);
      COPY (stringInfo.buffer^, str);
    END GetString;

  BEGIN (* GetAttrs *)
    GetString (dlg.gadget[SymSearchID], OLSettings.SymSearch);
    GetString (dlg.gadget[ObjSearchID], OLSettings.ObjSearch);
    GetString (dlg.gadget[WithPathID], OLSettings.WithPath);
    GetString (dlg.gadget[ProgPathID], OLSettings.ProgPath);
    GetString (dlg.gadget[SymExtID], OLSettings.SymExt);
    GetString (dlg.gadget[ObjExtID], OLSettings.ObjExt);
    GetString (dlg.gadget[WithExtID], OLSettings.WithExt);
    GetString (dlg.gadget[LinkCmdID], OLSettings.LinkCmd);
    GetString (dlg.gadget[LinkArgsID], OLSettings.LinkArgs);
    OLSettings.Verbose := (i.selected IN dlg.gadget[VerboseID].flags);
    OLSettings.MakeIcons := (i.selected IN dlg.gadget[MakeIconsID].flags);
    OLSettings.WithFmt := dlg.withFmt
  END GetAttrs;

(* ---------------------------------- *)
PROCEDURE (dlg : SettingsDialog) SetupWindow ( win : i.WindowPtr );
  BEGIN (* SetupWindow *)
    dlg.SetupWindow^ (win);
    dlg.SetAttrs
  END SetupWindow;

(* ---------------------------------- *)
PROCEDURE (dlg : SettingsDialog) HandleGadgetUp
  ( msg : i.IntuiMessagePtr )
  : INTEGER;

  VAR
    result, gadgetID : INTEGER;
    gadget : i.GadgetPtr;
    stringInfo : i.StringInfoPtr;
    code : INTEGER;
    drawer : ARRAY 256 OF CHAR;
    file : ARRAY 32 OF CHAR;
    buffer, str : e.LSTRPTR;

  (* ---------------------------------- *)
  PROCEDURE DoAbout;

    VAR es : i.EasyStruct;

    BEGIN (* DoAbout *)
      es.structSize := SIZE (i.EasyStruct);
      es.flags := {};
      es.title := cat.GetString (cat.gui3);
      es.gadgetFormat := cat.GetString (cat.gui4);
      es.textFormat := SYS.ADR ("%s (%s)\n%s");
      IF 0 = i.EasyRequest ( NIL, SYS.ADR (es), NIL,
          SYS.ADR (OLPrefsRev.vers), SYS.ADR (OLPrefsRev.date),
          cat.GetString (cat.gui5),
          NIL )
      THEN
      END
    END DoAbout;

  (* ---------------------------------- *)
  PROCEDURE DoSaveAs;

    VAR
      drawer : ARRAY 256 OF CHAR; file : ARRAY 32 OF CHAR;
      str : e.LSTRPTR;

    BEGIN (* DoSaveAs *)
      COPY (OLSettings.PrefsFile, drawer);
      str := d.PathPart (drawer);
      str[0] := 0X;

      str := d.FilePart (OLSettings.PrefsFile);
      COPY (str^, file);

      dlg.Block;
      str := cat.GetString (cat.gui5);
      IF RequestFile ( dlg.win, str^, "#?.prefs",
                       drawer, file, TRUE )
      THEN
        dlg.GetAttrs;
        IF ( d.AddPart (drawer, file, LEN (drawer))
             & OLSettings.SavePrefs (drawer) )
        THEN
          gt.SetGadgetAttrs ( dlg.gadget[PrefsFileID]^, dlg.win, NIL,
              gt.txText, SYS.ADR (OLSettings.PrefsFile),
              u.done );
          MakeIcon (drawer)
        ELSE
          Notice (cat.gui7)
        END
      END;
      dlg.UnBlock;
    END DoSaveAs;

  BEGIN (* HandleGadgetUp *)
    result := Events.Continue;

    code := msg.code;
    gadget := msg.iAddress;
    gt.ReplyIMsg (msg);

    gadgetID := gadget.gadgetID;
    CASE gadgetID OF
      BrowsePrefsID:
        COPY (OLSettings.PrefsFile, drawer);
        str := d.PathPart (drawer);
        str[0] := 0X;

        str := d.FilePart (OLSettings.PrefsFile);
        COPY (str^, file);

        dlg.Block;
        str := cat.GetString (cat.gui6);
        IF RequestFile ( dlg.win, str^, "#?.prefs",
                         drawer, file, FALSE )
        THEN
          IF d.AddPart (drawer, file, LEN (drawer))
           & OLSettings.LoadPrefs (drawer) THEN
            dlg.SetAttrs
          ELSE
            Notice (cat.gui7)
          END
        END;
        dlg.UnBlock
    |
      BrowseLinkerID:
        stringInfo := SYS.VAL ( i.StringInfoPtr,
            dlg.gadget[LinkCmdID].specialInfo );
        buffer := stringInfo.buffer;
        IF (buffer # NIL) & (buffer^ # "") THEN
          COPY (buffer^, drawer);
          str := d.PathPart (drawer);
          str[0] := 0X;

          str := d.FilePart (buffer^);
          COPY (str^, file);
        ELSE
          drawer := ""; file := ""
        END;

        dlg.Block;
        str := cat.GetString (cat.gui11);
        IF RequestFile ( dlg.win, str^, "#?",
                         drawer, file, FALSE )
        THEN
          IF d.AddPart (drawer, file, LEN (drawer)) THEN
            gt.SetGadgetAttrs ( dlg.gadget[LinkCmdID]^, dlg.win, NIL,
                gt.stString, SYS.ADR (drawer),
                u.done )
          END
        END;
        dlg.UnBlock
    |
      AboutID:
        DoAbout
    |
      SaveID:
        dlg.GetAttrs;
        IF OLSettings.SavePrefs (OLSettings.PrefsFile) THEN
          MakeIcon (OLSettings.PrefsFile)
        ELSE
          Notice (cat.gui8)
        END
    |
      SaveAsID:
        DoSaveAs()
    |
      QuitID:
        result := Events.Stop
    |
    ELSE (* NOP *)
    END;

    RETURN result
  END HandleGadgetUp;

(* ---------------------------------- *)
PROCEDURE (dlg : SettingsDialog) HandleGadgetDown
  ( msg : i.IntuiMessagePtr )
  : INTEGER;

  VAR
    result, gadgetID : INTEGER;
    gadget : i.GadgetPtr;
    code : INTEGER;

  BEGIN (* HandleGadgetDown *)
    result := Events.Continue;

    code := msg.code;
    gadget := msg.iAddress;
    gt.ReplyIMsg (msg);

    gadgetID := gadget.gadgetID;
    CASE gadgetID OF
      WithFmtID:
        dlg.withFmt := code
    |
    ELSE (* NOP *)
    END;

    RETURN result
  END HandleGadgetDown;

(* ---------------------------------- *)
PROCEDURE (dlg : SettingsDialog) InitObjects () : BOOLEAN;

  CONST
    template1 = "000000000000";
    template2 = "000000";
    template3 = "00000000000000000000000000000000";

  VAR
    obj : ARRAY 17 OF ea.OPTR;
    x, w1, w2, w3, ignore, hPlus2, vDiv2 : LONGINT;
    gad : ARRAY 20 OF SYS.ADDRESS; ta : SYS.ADDRESS;

  PROCEDURE StringGroup
    ( title     : e.LSTRPTR;
      gadget    : SYS.ADDRESS;
      minWidth  : LONGINT )
    : ea.OPTR;

    BEGIN (* StringGroup *)
      RETURN eam.HGroup (
          ea.Child,           eam.EmptyBox (1, u.done),
          ea.Child,           eam.GTString ( title,
              ea.GTTagList,       SYS.ADR (tags1),
              ea.InstanceAddress, gadget,
              ea.MinWidth,        minWidth,
              ea.Weight,          1,
              u.done ),
          u.done );
    END StringGroup;

  BEGIN
    FOR x := 0 TO LEN (dlg.gadget) - 1 DO
      gad[x] := SYS.ADR (dlg.gadget[x])
    END;
    ta := SYS.ADR (textAttr);

    w1 := ea.TextLength (textAttr, template1, 0X);
    w2 := ea.TextLength (textAttr, template2, 0X);
    w3 := ea.TextLength (textAttr, template3, 0X);

    hPlus2 := hSpace + 2;
    vDiv2 := vSpace DIV 2;

    (* set up some defaults for all objects *)
    ignore := ea.SetAttr ( NIL, ea.DefGTTextAttr, ta );

    (* build the object tree *)

    obj[0] := lbl.NewLabel ( dlg.label[0],
        ea.BorderTop,   1,
        ea.BorderRight, hSpace,
        u.done );
    lbl.InitLabel ( dlg.label[0], cat.GetString (cat.gui12),
        ta, gfx.jam1, {lbl.ShadowText}, dlg.drawInfo );
    obj[1] := eam.GTText ( NIL,
        ea.InstanceAddress, gad[PrefsFileID],
        ea.Weight,          1,
        u.done );
    obj[2] := eam.GTButton ( SYS.ADR ("..."),
        ea.ID,              BrowsePrefsID,
        u.done );

    obj[3] := eam.HGroup (
        ea.BorderLeft,   hPlus2,
        ea.BorderRight,  2,
        ea.BorderTop,    1,
        ea.BorderBottom, 1,
        ea.Child,        frm.NewFrame ( dlg.frame[0], u.done ),
        ea.Child,        obj[0],
        ea.Child,        obj[1],
        ea.Child,        obj[2],
        u.done );
    frm.InitFrame ( dlg.frame[0],
        {frm.Recessed}, dlg.visualInfo, dlg.drawInfo );

    (*
    ignore := ea.NewRelation ( obj[3], spt.SameHeightHook,
        ea.Object, obj[0],
        ea.Object, obj[1],
        ea.Object, obj[2],
        u.done );
    *)

    obj[4] := eam.VGroup (
        ea.BorderLeft,   hPlus2,
        ea.BorderRight,  hPlus2,
        ea.BorderTop,    vSpace,
        ea.BorderBottom, vSpace,
        ea.Child,        frm.NewFrame ( dlg.frame[1], u.done ),
        ea.Child,        lbl.NewLabel ( dlg.label[1],
            ea.BorderBottom, vDiv2,
            u.done ),
        ea.Child,        StringGroup ( cat.GetString (cat.gui13),
            gad[SymSearchID], w3 + 10 ),
        ea.Child,        StringGroup ( cat.GetString (cat.gui14),
            gad[ObjSearchID], w3 + 10 ),
        u.done );
    frm.InitFrame ( dlg.frame[1],
        {frm.DoubleBorder, frm.Recessed}, dlg.visualInfo, dlg.drawInfo );
    lbl.InitLabel ( dlg.label[1],
        cat.GetString (cat.gui15), ta,
        gfx.jam1, {lbl.ShadowText}, dlg.drawInfo );

    obj[5] := eam.VGroup (
        ea.BorderLeft,   hPlus2,
        ea.BorderRight,  hPlus2,
        ea.BorderTop,    vSpace,
        ea.BorderBottom, vSpace,
        ea.Weight,       100,
        ea.Child,        frm.NewFrame ( dlg.frame[2], u.done ),
        ea.Child,        lbl.NewLabel ( dlg.label[2],
            ea.BorderBottom, vDiv2,
            u.done ),
        ea.Child,        eam.HGroup (
            ea.Weight,       1,
            ea.Child,        eam.EmptyBox ( 1, u.done ),
            ea.Child,        eam.VGroup (
                ea.Child,        eam.EmptyBox (1, u.done),
                ea.Child,        StringGroup ( cat.GetString (cat.gui16),
                    gad[WithPathID], w1 + 10 ),
                ea.Child,        StringGroup ( cat.GetString (cat.gui17),
                    gad[ProgPathID], w1 + 10 ),
                ea.Child,        eam.EmptyBox (1, u.done),
                u.done ),
            ea.Child,        eam.EmptyBox ( 1, u.done ),
            u.done ),
        u.done );
    frm.InitFrame ( dlg.frame[2],
        {frm.DoubleBorder, frm.Recessed}, dlg.visualInfo, dlg.drawInfo );
    lbl.InitLabel ( dlg.label[2],
        cat.GetString (cat.gui18), ta,
        gfx.jam1, {lbl.ShadowText}, dlg.drawInfo );

    obj[6] := eam.VGroup (
        ea.BorderLeft,   hPlus2,
        ea.BorderRight,  hPlus2,
        ea.BorderTop,    vSpace,
        ea.BorderBottom, vSpace,
        ea.Weight,       100,
        ea.Child,        frm.NewFrame ( dlg.frame[3], u.done ),
        ea.Child,        lbl.NewLabel ( dlg.label[3],
            ea.BorderBottom, vDiv2,
            u.done ),
        ea.Child,        eam.HGroup (
            ea.Child,        eam.EmptyBox ( 1, u.done ),
            ea.Child,        eam.VGroup (
                ea.Child,        StringGroup ( cat.GetString (cat.gui19),
                    gad[SymExtID], w1 + 10 ),
                ea.Child,        StringGroup ( cat.GetString (cat.gui20),
                    gad[ObjExtID], w1 + 10 ),
                ea.Child,        StringGroup ( cat.GetString (cat.gui21),
                    gad[WithExtID], w1 + 10 ),
                u.done ),
            ea.Child,        eam.EmptyBox ( 1, u.done ),
            u.done ),
        u.done );
    frm.InitFrame ( dlg.frame[3],
        {frm.DoubleBorder, frm.Recessed}, dlg.visualInfo, dlg.drawInfo );
    lbl.InitLabel ( dlg.label[3],
        cat.GetString (cat.gui22), ta,
        gfx.jam1, {lbl.ShadowText}, dlg.drawInfo );

    txt[0] := cat.GetString (cat.gui23);
    txt[1] := cat.GetString (cat.gui24);
    obj[7] := eam.VGroup (
        ea.BorderLeft,   hPlus2,
        ea.BorderRight,  hPlus2,
        ea.BorderTop,    vSpace,
        ea.BorderBottom, vSpace,
        ea.Child, frm.NewFrame ( dlg.frame[5], u.done ),
        ea.Child, lbl.NewLabel ( dlg.label[5],
            ea.BorderBottom, vDiv2,
            u.done ),
        ea.Child,      eam.GTCheckBox ( txt[0],
            ea.GTFlags,         {gt.placeTextRight},
            ea.InstanceAddress, gad[VerboseID],
            ea.Weight,          1,
            u.done ),
        ea.Child,      eam.GTCheckBox ( txt[1],
            ea.GTFlags,         {gt.placeTextRight},
            ea.InstanceAddress, gad[MakeIconsID],
            ea.Weight,          1,
            u.done ),
        u.done );
    frm.InitFrame ( dlg.frame[5], {frm.DoubleBorder, frm.Recessed},
                    dlg.visualInfo, dlg.drawInfo );
    lbl.InitLabel ( dlg.label[5],
        cat.GetString (cat.gui25), ta, gfx.jam1,
        {lbl.ShadowText}, dlg.drawInfo );

    txt[0] := cat.GetString (cat.gui34);
    txt[1] := cat.GetString (cat.gui35);
    txt[2] := NIL;
    (* txt[2] := cat.GetString (cat.gui36); *)
    (* txt[3] := NIL; *)
    tags2[0].tag := gt.mxLabels; tags2[0].data := SYS.ADR (txt);
    tags2[1].tag := u.done;
    obj[8] := eam.VGroup (
        ea.BorderLeft,   hPlus2,
        ea.BorderRight,  hPlus2,
        ea.BorderTop,    vSpace,
        ea.BorderBottom, vSpace,
        ea.Child,        frm.NewFrame ( dlg.frame[4], u.done ),
        ea.Child,        lbl.NewLabel ( dlg.label[4],
            ea.BorderBottom, vDiv2,
            u.done ),
        ea.Child,        eam.HGroup (
            ea.Child,        eam.VGroup (
                ea.Weight,       1,
                ea.Child,        eam.HGroup (
                    ea.Child, eam.GTText ( cat.GetString (cat.gui26),
                        ea.Weight,          1,
                        u.done ),
                    ea.Child, eam.GTButton ( SYS.ADR ("..."),
                        ea.ID,              BrowseLinkerID,
                        u.done ),
                    ea.Child, eam.GTString ( NIL,
                        ea.GTTagList,       SYS.ADR (tags1),
                        ea.InstanceAddress, gad[LinkCmdID],
                        ea.MinWidth,        w3 + 10,
                        u.done ),
                    u.done ),
                ea.Child,        StringGroup ( cat.GetString (cat.gui27),
                    gad[LinkArgsID], w3 + 10 ),
                u.done ),
            ea.Child,        eam.EmptyBox (1, ea.MinWidth, hSpace, u.done),
            ea.Child,        eam.GTMX (
                ea.Weight,          1,
                ea.GTFlags,         {gt.placeTextRight},
                ea.GTTagList,       SYS.ADR (tags2),
                ea.ID,              WithFmtID,
                ea.InstanceAddress, gad[WithFmtID],
                u.done ),
            u.done ),
        u.done );
    frm.InitFrame ( dlg.frame[4],
        {frm.DoubleBorder, frm.Recessed}, dlg.visualInfo, dlg.drawInfo );
    lbl.InitLabel ( dlg.label[4],
        cat.GetString (cat.gui28), ta,
        gfx.jam1, {lbl.ShadowText}, dlg.drawInfo );

    obj[9] := eam.GTButton ( cat.GetString (cat.gui29),
        ea.ID,           AboutID,
        ea.Weight,       1,
        u.done );
    obj[10] := eam.GTButton ( cat.GetString (cat.gui30),
        ea.ID,           SaveID,
        ea.Weight,       1,
        u.done );
    obj[11] := eam.GTButton ( cat.GetString (cat.gui31),
        ea.ID,           SaveAsID,
        ea.Weight,       1,
        u.done );
    obj[12] := eam.GTButton ( cat.GetString (cat.gui32),
        ea.ID,           QuitID,
        ea.Weight,       1,
        u.done );
    obj[13] := eam.HGroup (
        (* ea.BorderTop,    vSpace, *)
        ea.BorderBottom, vSpace,
        ea.Child,        eam.EmptyBox (1, ea.MinWidth, hSpace, u.done),
        ea.Child,        obj[9],
        ea.Child,        eam.EmptyBox (1, ea.MinWidth, hSpace, u.done),
        ea.Child,        obj[10],
        ea.Child,        eam.EmptyBox (1, ea.MinWidth, hSpace, u.done),
        ea.Child,        obj[11],
        ea.Child,        eam.EmptyBox (1, ea.MinWidth, hSpace, u.done),
        ea.Child,        obj[12],
        ea.Child,        eam.EmptyBox (1, ea.MinWidth, hSpace, u.done),
        u.done );
    ignore := ea.NewRelation ( obj[13], spt.SameSizeHook,
        ea.Object, obj[9],
        ea.Object, obj[10],
        ea.Object, obj[11],
        ea.Object, obj[12],
        u.done );

    dlg.winObj := eam.VGroup (
        ea.Child,        eam.VGroup (
            ea.BorderLeft,   hSpace,
            ea.BorderRight,  hSpace,
            ea.BorderTop,    vSpace,
            ea.BorderBottom, vSpace,
            (* ea.Child,        frm.NewFrame ( dlg.frame[6], u.done ), *)
            ea.Child,        obj[3],
            ea.Child,        eam.EmptyBox (1, ea.MinHeight, vSpace, u.done),
            ea.Child,        eam.HGroup (
                ea.Child,        obj[4],
                ea.Child,        eam.EmptyBox (1, ea.MinWidth, hSpace, u.done),
                ea.Child,        obj[7],
                u.done ),
            ea.Child,        eam.EmptyBox (1, ea.MinHeight, vSpace, u.done),
            ea.Child,        eam.HGroup (
                ea.Child,        obj[5],
                ea.Child,        eam.EmptyBox (1, ea.MinWidth, hSpace, u.done),
                ea.Child,        obj[6],
                u.done ),
            ea.Child,        eam.EmptyBox (1, ea.MinHeight, vSpace, u.done),
            ea.Child,        obj[8],
            u.done ),
        ea.Child,        obj[13],
        u.done );
    (* frm.InitFrame ( dlg.frame[6], {}, dlg.visualInfo, dlg.drawInfo ); *)

    RETURN (dlg.winObj # NIL)
  END InitObjects;

(* ---------------------------------- *)
PROCEDURE (dlg : SettingsDialog) OpenWindow () : BOOLEAN;

  BEGIN
    dlg.title := cat.GetString (cat.gui1);
    dlg.flags := {i.windowDrag, i.windowDepth, i.windowClose, i.activate};
    dlg.IDCMP := {i.closeWindow, i.refreshWindow}
                 + gt.buttonIDCMP + gt.stringIDCMP
                 + gt.checkBoxIDCMP + gt.mxIDCMP;

    RETURN dlg.OpenWindow^()
  END OpenWindow;

(* ---------------------------------- *)
PROCEDURE* Cleanup ( VAR rc : LONGINT );

  BEGIN (* Cleanup *)
    IF settingsDlg # NIL THEN
      settingsDlg.Close; settingsDlg := NIL
    END;

    IF drawInfo # NIL THEN
      i.FreeScreenDrawInfo (scr, drawInfo);
      drawInfo := NIL;
    END;

    IF visualInfo # NIL THEN
      gt.FreeVisualInfo (visualInfo);
      visualInfo := NIL;
    END;

    IF scr # NIL THEN
      i.UnlockPubScreen (e.NILSTR, scr);
      scr := NIL;
    END;
  END Cleanup;


(* ---------------------------------- *)
PROCEDURE Open*;

  (* ---------------------------------- *)
  PROCEDURE InitDisplay;

    VAR
      scrFont : gfx.TextFontPtr;

    BEGIN (* InitDisplay *)
      (* lock the screen *)
      scr := i.LockPubScreen (e.NILSTR);
      ASSERT (scr # NIL, Errors.postCondition);

      (* get VisualInfo and DrawInfo *)
      visualInfo := gt.GetVisualInfo (scr, u.done);
      drawInfo := i.GetScreenDrawInfo (scr);
      ASSERT ((visualInfo # NIL) & (drawInfo # NIL), Errors.postCondition);

      (* Calculate spacing from screen dimensions *)
      hSpace := scr.width DIV 100;
      vSpace := scr.height DIV 84;

      (* get the font(s) *)
      scrFont := drawInfo.font;
      SYS.NEW ( textAttr.name,
                str.Length (scrFont.message.node.name^) + 1 );
      COPY (scrFont.message.node.name^, textAttr.name^);
      textAttr.ySize := scrFont.ySize;
      textAttr.style := scrFont.style;
      textAttr.flags := scrFont.flags;
    END InitDisplay;

  BEGIN (* Open *)
    ASSERT (ea.base # NIL, Errors.noLibrary);
    ASSERT (settingsDlg = NIL, Errors.preCondition);
    Kernel.SetCleanup (Cleanup);
    InitDisplay;
    NEW (settingsDlg);
    IF settingsDlg.Open (visualInfo, drawInfo) THEN
      Events.SimpleLoop (settingsDlg, Events.NoGC);
      settingsDlg.Close; settingsDlg := NIL
    END
  END Open;

BEGIN
  tags1[0].tag := gt.stMaxChars; tags1[0].data := OLSettings.StrLen;
  tags1[1].tag := u.done;
END OLPrefsGUI.

(*************************************************************************

  $Log: OLPrefsGUI.mod $
# Revision 1.2  1995/06/29  19:27:47  fjc
# - Uses new interface to Events.SimpleLoop.
#
# Revision 1.1  1995/06/04  23:07:03  fjc
# Initial revision
#
*************************************************************************)
